@@ -44,7 +44,9 @@ namespace Twitterizer.Commands
     /// Returns the un-favorited status in the requested format when successful.
     /// </summary>
     [AuthorizedCommandAttribute]
+#if !SILVERLIGHT
     [Serializable]
+#endif
     internal sealed class DeleteFavoriteCommand : TwitterCommand<TwitterStatus>
     {
         /// <summary>
@@ -54,7 +56,7 @@ namespace Twitterizer.Commands
         /// <param name="statusId">The status id.</param>
         /// <param name="options">The options.</param>
         public DeleteFavoriteCommand(OAuthTokens tokens, decimal statusId, OptionalProperties options)
-            : base(HTTPVerb.POST, "/favorites/destroy.json", tokens, options)
+            : base(HTTPVerb.POST, string.Format(CultureInfo.InvariantCulture.NumberFormat, "favorites/destroy.json?id={0}", statusId), tokens, options)
         {
             if (statusId <= 0)
             {
@@ -65,22 +67,13 @@ namespace Twitterizer.Commands
             {
                 throw new ArgumentNullException("tokens");
             }
-
-            this.StatusId = statusId;
         }
 
-        /// <summary>
-        /// Gets or sets the status id.
-        /// </summary>
-        /// <value>The status id.</value>
-        public decimal StatusId { get; set; }
-
         /// <summary>
         /// Initializes the command.
         /// </summary>
         public override void Init()
         {
-            this.RequestParameters.Add("id", this.StatusId.ToString(CultureInfo.InvariantCulture));
         }
     }
 }
