@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2015 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -137,6 +131,9 @@ namespace Smuxi.Engine
                 if (targetType == typeof(string[])) {
                     return (T)(object) GetList(key);
                 }
+                if (targetType == typeof(MessageBufferPersistencyType)) {
+                    return (T) Enum.Parse(typeof(MessageBufferPersistencyType), strValue, true);
+                }
                 // handle empty booleans and integers
                 if (targetType.IsValueType && String.IsNullOrEmpty(strValue)) {
                     return default(T);
@@ -244,7 +241,11 @@ namespace Smuxi.Engine
             Get(prefix+"Port", 7689);
             Get(prefix+"Channel", "TCP");
             Get(prefix+"Formatter", "binary");
-            
+
+            prefix = "Engine/";
+            var oldConfigVersion = Get<string>(prefix+"ConfigVersion", null);
+            Get(prefix+"ConfigVersion", Engine.AssemblyVersion.ToString());
+
             prefix = "Engine/Users/DEFAULT/Interface/";
             Get(prefix+"ShowAdvancedSettings", false);
 
@@ -276,7 +277,8 @@ namespace Smuxi.Engine
             Get(prefix+"FontStyle",  String.Empty);
             Get(prefix+"FontSize",   0);
             Get(prefix+"WrapMode",   "Word");
-            
+            Get(prefix+"Emojis", false);
+
             prefix = "Engine/Users/DEFAULT/Interface/Entry/";
             Get(prefix+"CompletionCharacter", ":");
             Get(prefix+"CommandCharacter", "/");
@@ -305,7 +307,8 @@ namespace Smuxi.Engine
             Get(prefix+"LogFilteredMessages", false);
 
             prefix = "Engine/Users/DEFAULT/MessageBuffer/";
-            Get(prefix+"PersistencyType", "Volatile");
+            Get<string>(prefix+"PersistencyType",
+                        MessageBufferPersistencyType.PersistentSqlite.ToString());
             prefix = "Engine/Users/DEFAULT/MessageBuffer/Volatile/";
             Get(prefix+"MaxCapacity", 200);
             prefix = "Engine/Users/DEFAULT/MessageBuffer/Persistent/";
@@ -325,6 +328,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.oftc.net");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "OFTC");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -340,6 +345,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.gimp.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "GIMPNet");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -349,6 +356,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.geekshed.net");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "GeekShed");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -358,6 +367,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.efnet.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "EFnet");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -367,6 +378,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.ircnet.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "IRCnet");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -376,6 +389,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.freenode.net");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "freenode");
+            Get(prefix + "Nickname", String.Empty);
+            Get(prefix + "Realname", String.Empty);
             Get(prefix + "Username", String.Empty);
             Get(prefix + "Password", String.Empty);
             Get(prefix + "UseEncryption", false);
@@ -401,6 +416,14 @@ namespace Smuxi.Engine
             // loading defaults
             LoadAllEntries("Engine/Users/DEFAULT");
                     
+            // are we upgrading from <= 1.0?
+            if (String.IsNullOrEmpty(oldConfigVersion) ||
+                new Version(oldConfigVersion) < new Version(1, 0)) {
+                // change default value to PersistentSqlite
+                SetUserEntry("DEFAULT", "MessageBuffer/PersistencyType",
+                             MessageBufferPersistencyType.PersistentSqlite.ToString());
+            }
+
             prefix = "Engine/Users/";
             string[] users = GetList(prefix+"Users");
             m_Preferences[prefix + "Users"] = users;
@@ -446,7 +469,7 @@ namespace Smuxi.Engine
 #endif
                 }
                 if (String.IsNullOrEmpty(realname)) {
-                    realname = "http://www.smuxi.org/";
+                    realname = "Your Name";
                 }
                 LoadUserEntry(user, "Connection/Realname", realname);
                 LoadUserEntry(user, "Connection/Encoding", String.Empty);
@@ -505,6 +528,7 @@ namespace Smuxi.Engine
                 LoadUserEntry(user, "Interface/Chat/FontStyle", null);
                 LoadUserEntry(user, "Interface/Chat/FontSize", null);
                 LoadUserEntry(user, "Interface/Chat/WrapMode", null);
+                LoadUserEntry(user, "Interface/Chat/Emojis", null);
                 LoadUserEntry(user, "Interface/Entry/CompletionCharacter", null);
                 LoadUserEntry(user, "Interface/Entry/CommandCharacter", null);
                 LoadUserEntry(user, "Interface/Entry/BashStyleCompletion", null);
@@ -518,7 +542,16 @@ namespace Smuxi.Engine
                 LoadUserEntry(user, "Logging/Enabled", null);
                 LoadUserEntry(user, "Logging/LogFilteredMessages", null);
 
-                LoadUserEntry(user, "MessageBuffer/PersistencyType", null);
+                var persistencyKey = "MessageBuffer/PersistencyType";
+                LoadUserEntry(user, persistencyKey , null);
+                // are we upgrading from < 1.0?
+                if (String.IsNullOrEmpty(oldConfigVersion) ||
+                    new Version(oldConfigVersion) < new Version(1, 0)) {
+                    // migrate all users automatically to SQLite
+                    SetUserEntry(user, persistencyKey,
+                                 MessageBufferPersistencyType.PersistentSqlite.ToString());
+                }
+
                 LoadUserEntry(user, "MessageBuffer/Volatile/MaxCapacity", null);
                 LoadUserEntry(user, "MessageBuffer/Persistent/MaxCapacity", null);
 
@@ -545,6 +578,10 @@ namespace Smuxi.Engine
                                       Get(dprefix + server + "/Encoding", null));
                             LoadEntry(sprefix + server + "/AutoConvertUTF8",
                                       Get(dprefix + server + "/AutoConvertUTF8", null));
+                            LoadEntry(sprefix + server + "/Nickname",
+                                      Get(dprefix + server + "/Nickname", null));
+                            LoadEntry(sprefix + server + "/Realname",
+                                      Get(dprefix + server + "/Realname", null));
                             LoadEntry(sprefix + server + "/Username",
                                       Get(dprefix + server + "/Username", null));
                             LoadEntry(sprefix + server + "/Password",
@@ -571,6 +608,8 @@ namespace Smuxi.Engine
                     LoadEntry(sprefix+"Network", String.Empty);
                     LoadEntry(sprefix+"Encoding", null);
                     LoadEntry(sprefix+"AutoConvertUTF8", null);
+                    LoadEntry(sprefix+"Nickname", String.Empty);
+                    LoadEntry(sprefix+"Realname", String.Empty);
                     LoadEntry(sprefix+"Username", String.Empty);
                     LoadEntry(sprefix+"Password", String.Empty);
                     LoadEntry(sprefix+"UseEncryption", false);
@@ -606,6 +645,22 @@ namespace Smuxi.Engine
                     LoadUserEntry(user, cprefix + "MessageType", null);
                     LoadUserEntry(user, cprefix + "MessagePattern", null);
                 }
+
+                string lprefix = "MessagePatterns/";
+                var linkKeys = GetList(prefix + user + "/" + lprefix + "MessagePatterns");
+                if (linkKeys == null) {
+                    linkKeys = new string[] {};
+                    m_Preferences[prefix + user + "/" + lprefix + "MessagePatterns"] = new string[] {};
+                } else {
+                    m_Preferences[prefix + user + "/" + lprefix + "MessagePatterns"] = linkKeys;
+                }
+                foreach (var linkKey in linkKeys) {
+                    lprefix = "MessagePatterns/" + linkKey + "/";
+                    LoadUserEntry(user, lprefix + "MessagePartPattern", String.Empty);
+                    LoadUserEntry(user, lprefix + "MessagePartType", String.Empty);
+                    LoadUserEntry(user, lprefix + "LinkFormat", String.Empty);
+                    LoadUserEntry(user, lprefix + "TextFormat", String.Empty);
+                }
             }
         }
 
@@ -681,22 +736,52 @@ namespace Smuxi.Engine
             }
         }
 
+
+        protected string GetUserKey(string user, string key)
+        {
+            var prefix = "Engine/Users/";
+            var ukey = prefix+user+"/"+key;
+            return ukey;
+        }
+
+        [Obsolete]
+        protected object GetUserEntry(string user, string key, object defaultvalue)
+        {
+            var ukey = GetUserKey(user, key);
+            return Get(ukey, defaultvalue);
+        }
+
+        protected T GetUserEntry<T>(string user, string key, T defaultvalue)
+        {
+            var ukey = GetUserKey(user, key);
+            return Get<T>(ukey, defaultvalue);
+        }
+
+        protected void SetUserEntry(string user, string key, object value)
+        {
+#if CONFIG_DEBUG
+            Trace.Call(user, key, value);
+#endif
+            var ukey = GetUserKey(user, key);
+            m_Preferences[ukey] = value;
+        }
+
         protected void LoadUserEntry(string user, string key, object defaultvalue)
         {
 #if CONFIG_DEBUG
             Trace.Call(user, key, defaultvalue);
 #endif
 
-            string prefix = "Engine/Users/";
-            string ukey = prefix+user+"/"+key;
             object obj;
             if (defaultvalue is string) {
-                obj = Get<string>(ukey, (string) defaultvalue);
+                obj = GetUserEntry<string>(user, key, (string) defaultvalue);
+            } else if (defaultvalue is bool) {
+                obj = GetUserEntry<bool>(user, key, (bool) defaultvalue);
             } else {
-                obj = Get(ukey, defaultvalue);
+                obj = GetUserEntry(user, key, defaultvalue);
             }
             if (obj != null) {
-                m_Preferences[ukey] = obj;
+                SetUserEntry(user, key, obj);
             }
         }
         
