@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2014 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -38,12 +38,14 @@ namespace Smuxi.Frontend.Gnome
         private static readonly log4net.ILog f_Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
         private bool             _IsFullscreen;
+        string f_NetworkStatus;
 
         Gtk.Statusbar NetworkStatusbar { get; set; }
         Gtk.Statusbar Statusbar { get; set; }
         public Gtk.ProgressBar ProgressBar { get; private set; }
         Gtk.HBox StatusHBox { get; set; }
         public MenuWidget MenuWidget { get; private set; }
+        Gtk.HPaned TreeViewHPaned { get; set; }
 
         public IFrontendUI UI { get; private set; }
         public Entry Entry { get; private set; }
@@ -67,6 +69,7 @@ namespace Smuxi.Frontend.Gnome
         public NotificationAreaIconMode NotificationAreaIconMode { get; set; }
         public bool IsMinimized { get; private set; }
         public bool IsMaximized { get; private set; }
+        public int WindowWidth { get; private set; }
 
         public bool CaretMode {
             get {
@@ -99,6 +102,7 @@ namespace Smuxi.Frontend.Gnome
                 }
                 NetworkStatusbar.Pop(0);
                 NetworkStatusbar.Push(0, value);
+                f_NetworkStatus = value;
             }
         } 
 
@@ -259,9 +263,14 @@ namespace Smuxi.Frontend.Gnome
                 treeviewScrolledWindow.CheckResize();
             };
 
+            var notebookPaned = new Gtk.VPaned();
+            notebookPaned.Pack1(Notebook, true, false);
+            notebookPaned.Pack2(entryScrolledWindow, false, false);
+
             var treeviewPaned = new Gtk.HPaned();
             treeviewPaned.Pack1(treeviewScrolledWindow, false, false);
-            treeviewPaned.Pack2(Notebook, true, false);
+            treeviewPaned.Pack2(notebookPaned, true, false);
+            TreeViewHPaned = treeviewPaned;
 
             var entryPaned = new Gtk.VPaned();
             entryPaned.ButtonPressEvent += (sender, e) => {
@@ -275,7 +284,6 @@ namespace Smuxi.Frontend.Gnome
                 }
             };
             entryPaned.Pack1(treeviewPaned, true, false);
-            entryPaned.Pack2(entryScrolledWindow, false, false);
 
             Gtk.VBox vbox = new Gtk.VBox();
             vbox.PackStart(MenuWidget, false, false, 0);
@@ -349,6 +357,9 @@ namespace Smuxi.Frontend.Gnome
             if (chatView == null) {
                 chatView = ChatViewManager.CurrentChatView;
             }
+            if (protocolStatus == null) {
+                protocolStatus = f_NetworkStatus;
+            }
             if (chatView == null) {
                 return;
             }
@@ -358,6 +369,14 @@ namespace Smuxi.Frontend.Gnome
                 title = String.Empty;
             } else if (chatView is ProtocolChatView) {
                 title = protocolStatus;
+            } else if (chatView is GroupChatView) {
+                var groupChatView = (GroupChatView) chatView;
+                var users = String.Format(_("{0} Users"),
+                                          groupChatView.Participants.Count);
+                title = String.Format("{0} ({1}) @ {2}",
+                                      chatView.Name,
+                                      users,
+                                      protocolStatus);
             } else {
                 title = String.Format("{0} @ {1}",
                                       chatView.Name,
@@ -371,6 +390,15 @@ namespace Smuxi.Frontend.Gnome
             Title = title;
         }
 
+        protected override bool OnConfigureEvent(Gdk.EventConfigure e)
+        {
+            Trace.Call(e);
+
+            WindowWidth = e.Width;
+            CheckLayout();
+            return base.OnConfigureEvent(e);
+        }
+
         protected virtual void OnDeleteEvent(object sender, Gtk.DeleteEventArgs e)
         {
             Trace.Call(sender, e);
@@ -455,7 +483,7 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
 
-                chatView.OutputMessageTextView.UpdateMarkerline();
+                chatView.UpdateLastSeenMessage();
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -497,6 +525,10 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
                     f_Logger.Debug("OnWindowStateEvent(): IsMaximized: " + IsMaximized);
 #endif
+                    GLib.Idle.Add(() => {
+                        CheckLayout();
+                        return false;
+                    });
                 }
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
@@ -568,6 +600,16 @@ namespace Smuxi.Frontend.Gnome
                 }
                 Entry.GrabFocus();
             };
+            if (e.ChatView is GroupChatView) {
+                var groupChatView = (GroupChatView) e.ChatView;
+                groupChatView.ParticipantsChanged += (o, args) => {
+                    if (ChatViewManager.CurrentChatView != groupChatView) {
+                        return;
+                    }
+                    UpdateTitle(groupChatView, null);
+                };
+                groupChatView.OutputHPaned.Position = (WindowWidth / 7) * 5;
+            }
             UpdateProgressBar();
         }
         
@@ -613,6 +655,18 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        void CheckLayout()
+        {
+            TreeViewHPaned.Position = WindowWidth / 7;
+            foreach (var chat in ChatViewManager.Chats) {
+                if (!(chat is GroupChatView)) {
+                    continue;
+                }
+                var groupChat = (GroupChatView) chat;
+                groupChat.OutputHPaned.Position = (WindowWidth / 7) * 5;
+            }
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
